import io
import os
import json
from inventory import Inventory
from room import Room
from interactable import Interactable

class Game:
    def __init__(self) -> None:
        self.is_running   : bool       = True
        self.inventory    : Inventory  = None
        self.game_data    : dict       = None
        self.rooms        : list[Room] = []
        self.current_room : Room       = None

    def tiges(mett):
        pass
    
    def load_game(self, file: str) -> None:
        try:
            with open(file, 'r') as f:
                raw_game_data = f.read()
                self.game_data = json.loads(raw_game_data)

                self.inventory = Inventory(self.game_data["items"])

                if 'rooms' in self.game_data:
                    
                    for room_id in self.game_data['rooms']:
                        room = self.game_data['rooms'][room_id]

                        g_room = Room(room_id, room['name'])
                        g_room.identifier = room['identifier']
                        g_room.enter_text = room['enter_text']
                        g_room.doors = room['doors']
                        g_room.interactables = []
                        g_room.requires_key = room['requires_key'] if 'requires_key' in room else False
                        g_room.unlocked = g_room.requires_key == False
                        g_room.is_start = room['is_start'] if 'is_start' in room else False
                        
                        # Check if we had a starting room already
                        if g_room.is_start:
                            for r in self.rooms:
                                if r.is_start:
                                    print(f"Error: {g_room.name} is marked as the starting room, but {r.name} is already marked as the starting room.")
                                    os._exit(1)
                        
                        if 'interactables' in room:
                            for interactable in room['interactables']:
                                i = Interactable(self, g_room, interactable)
                                g_room.interactables.append(i)

                        self.rooms.append(g_room)

                self.current_room = self.get_start_room()
        except FileNotFoundError:
            print("File not found.")
    
    def get_rooms(self) -> list[Room]:
        if not self.rooms or len(self.rooms) == 0:
            return []
        else:
            return self.rooms
    
    def get_room(self, id : str) -> Room | None:
        for room in self.rooms:
            if room.id == id:
                return room
        
        return None
    
    def get_matching_rooms(self, query : str) -> list[Room]:
        rooms : list[Room] = []
        query : str        = query.lower().strip()

        for room in self.rooms:
            if room.id == id:
                rooms.append(room)
            else:
                for identifier in room.identifier:
                    if identifier.lower().strip().startswith(query):
                        already_there = False
                        for r in rooms:
                            if r.id == room.id:
                                already_there = True
                        
                        if not already_there:
                            rooms.append(room)
                        
        return rooms

    def get_start_room(self) -> Room | None:
        for room in self.rooms:
            if room.is_start:
                return room
        
        return None

    def run_actions(self, actions : str|list[str]) -> bool:
        if isinstance(actions, list) == False:
            actions = [ actions ]

        for action in actions:
            print(f"[DEBUG] run_action('{action}')")

            action_parts = action.lower().split(' ')
            action_cmd = action_parts[0]
            action_args = list(filter(None, action_parts[1:] if len(action_parts) >= 2 else []))

            if action_cmd == "set_room_flag":
                room = self.get_room(action_args[0])
                if room is not None:
                    room.set_flag(action_args[1])
                else:
                    print(f"[DEBUG] Error: {action} (Room not found)")
            
            elif action_cmd == "add_item":
                self.inventory.add_item(action_args[0])

            elif action_cmd == "remove_item":
                self.inventory.remove_item(action_args[0])

            else:
                return False