from interactable import Interactable

class Room:
    def __init__(self, id : str, name : str):
        self.id            : str                = id
        self.name          : str                = name
        self.identifier    : list[str]          = []
        self.enter_text    : str                = ""
        self.is_start      : bool               = False
        self.doors         : list[str]          = []
        self.interactables : list[Interactable] = []
        self.flags         : list               = []
        self.requires_key  : str|bool           = False
        self.unlocked      : bool               = True
    
    def set_flag(self, flag : str) -> None:
        if flag not in self.flags:
            self.flags.append(flag)
            print(f"[DEBUG] set_flag({flag}) in {self.name}")
    
    def has_flag(self, flag : str) -> bool:
        return flag in self.flags

    def get_interactable(self, name : str) -> Interactable|None:
        for i in self.interactables:
            if name.lower() in i.identifier:
                return i
        
        return None
    
    def is_adjacent(self, room) -> bool:
        if type(room) is not Room:
            print("[WARN] room is not of type Room")
            return False
        
        return room.id in self.doors

    def __str__(self):
        r = f"[Room] ID: {self.id} | Name: {self.name} | IsStart: {self.is_start}"

        for door in self.doors:
            r += f"\n\t- [Door] To: {door}"
        
        for interactable in self.interactables:
            r += f"\n\t- [Interactable] Name: {interactable.name} | Text: {interactable.text}"
        
        return r